#!/usr/bin/env bash

cd "$(dirname "$0")" || exit
# shellcheck disable=SC1091
source "$PWD/taskrunner/runner.sh"

VERSION="1.""$(git rev-list --count HEAD)"

set_version() {
  if ! (git update-index --refresh) ; then
      echo "uncommited changes"
      exit 1
  fi

  git tag "$VERSION"
  echo "$VERSION" > ./studentsweb/source/static/version
}

task_copy() {

  set_version

  rsync -azP \
    --exclude '.git' \
    --exclude='*/node_modules/' \
    --exclude='*.yarn-error' \
    "$PWD" superlinux@manticore.albertov.dev:/home/superlinux/Documents
}

# DEV section
task_build:dev() {
    docker-compose -f docker-compose.dev.yml build
}

task_run:dev() {
    docker-compose -f docker-compose.dev.yml up
}

task_stop:dev() {
    docker-compose -f docker-compose.dev.yml stop
}

# PROD section
task_build:prod() {
    docker-compose -f docker-compose.prod.yml build
}

task_run:prod() {
    docker-compose -f docker-compose.prod.yml up -d
}

task_stop:prod() {
    docker-compose -f docker-compose.prod.yml stop
}
