import ldap from "ldapjs";
import type { ReBody } from "./interfaces";

export async function tryToAuth(
  username: string,
  password: string
): Promise<ReBody> {
  return new Promise((resolve, reject) => {
    let found = false;
    // Anonymous bind
    var client = ldap.createClient({
      url: "ldap://ldap.fbmi.cvut.cz",
      strictDN: false,
    });

    const opts = {
      filter: `(&(objectclass=inetOrgPerson)(uid=${username}))`,
      scope: "sub",
      attributes: ["dn", "sn", "cn"],
    };

    client.bind("", "", function (error) {
      if (error) {
        resolve(getResponse(true, "offline ldap"));
      }
    });
    // Search for right DN
    client.search(`dc=fbmi,dc=cvut,dc=cz`, opts, (err, res) => {
      res.on("searchEntry", (entry) => {
        found = true;
        client.bind(entry.object.dn, password, function (error) {
          if (!error) {
            resolve(getResponse(true, "ok"));
          } else {
            resolve(getResponse(false, "bad username or password"));
          }
        });
      });

      res.on("error", (err) => {
        resolve(getResponse(false, "bad username or password"));
      });

      res.on("end", (res) => {
        if (!found) {
          resolve(getResponse(false, "username not found"));
        }
      });
    });
  });
}

function getResponse(boo: boolean, message: string): ReBody {
  return {
    body: { auth: boo, message: message },
  };
}
